import pytest
import pendulum
from pendulum import datetime

from mkdocs_footermatter.plugin import FootermatterPlugin
from mkdocs.config import config_options


class Vars:
    def __init__(self, dict):
        self._vars = dict

    def __getitem__(self, name):
        return self._vars[name]

    def __iter__(self):
        return iter(self._vars)

    def keys(self):
        return self._vars.keys()

    def items(self):
        return self._vars.items()

    def values(self):
        return self._vars.values()


class Page():
    def __init__(self, meta):
        self.meta = meta


@pytest.fixture
def plugin():
    plugin = FootermatterPlugin()
    plugin.load_config(options={})
    plugin.on_config(plugin.config)
    return plugin


def test_locale(plugin):
    # Ensure specified locale works
    cfg = config_options.BaseConfigOption().default
    plugin.config['locale'] = 'test'
    assert plugin.get_locale(cfg) == 'test'

    # Ensure language from theme works
    cfg = {'theme': Vars({'language': 'nb'})}
    plugin.config['locale'] = None
    assert plugin.get_locale(cfg) == 'nb'

    # Ensure locale from theme works
    cfg = {'theme': Vars({'locale': 'nn'})}
    plugin.config['locale'] = None
    assert plugin.get_locale(cfg) == 'nn'

    # Ensure fallback locale works
    cfg = {'': ''}
    plugin.config['locale'] = None
    assert plugin.get_locale(cfg) == 'en'


def test_plugin(plugin):
    page = Page({'authors': 'Test Author', 'created': datetime(2022, 1, 1), 'updated': datetime(2022, 1, 1)})
    context_after = plugin.on_page_context({}, page)
    assert context_after.get('footermatter_authors') is not None
    assert context_after.get('footermatter_authors')[0].name == 'Test Author'
    assert context_after.get('footermatter_authors')[0].img is plugin.config['default_author_img']
    assert context_after.get('footermatter_authors')[0].url is plugin.config['default_author_url']
    assert context_after.get('footermatter_created') is not None
    assert context_after.get('footermatter_updated') is not None

    plugin.config['author_map'] = ['Test Author | test_image | test_url']
    plugin.on_config(plugin.config)
    context_after = plugin.on_page_context({}, page)
    assert context_after.get('footermatter_authors')[0].img == 'test_image'
    assert context_after.get('footermatter_authors')[0].url == 'test_url'

    page = Page({})
    context_after = plugin.on_page_context({}, page)
    assert context_after.get('footermatter_authors') is None
    assert context_after.get('footermatter_created') is None
    assert context_after.get('footermatter_updated') is None

    page = Page({'created': datetime(2022, 1, 1), 'updated': datetime(2022, 1, 1)})
    context_after = plugin.on_page_context({}, page)
    assert context_after.get('footermatter_authors') is None
    assert context_after.get('footermatter_created') is not None
    assert context_after.get('footermatter_updated') is not None

    page = Page({'authors': 'Test Author'})
    context_after = plugin.on_page_context({}, page)
    assert context_after.get('footermatter_authors') is not None
    assert context_after.get('footermatter_created') is None
    assert context_after.get('footermatter_updated') is None


def test_formats(plugin):
    # Date
    plugin.config['date_format'] = 'date'
    page = Page({'created': '2022-02-01 10:00'})
    context_after = plugin.on_page_context({}, page)
    assert context_after.get('footermatter_created') == 'February 1, 2022'

    page = Page({'created': '01 feb.22'})
    context_after = plugin.on_page_context({}, page)
    assert context_after.get('footermatter_created') == 'February 1, 2022'

    page = Page({'created': 'feb 1 2022'})
    context_after = plugin.on_page_context({}, page)
    assert context_after.get('footermatter_created') == 'February 1, 2022'

    # Datetime
    plugin.config['date_format'] = 'datetime'
    plugin.config['locale'] = 'en'
    page = Page({'created': 'feb 1 2022'})
    context_after = plugin.on_page_context({}, page)
    assert context_after.get('footermatter_created') == 'February 1, 2022 12:00 AM'

    plugin.config['locale'] = 'nb'
    page = Page({'created': '2022-02-01 12:00'})
    context_after = plugin.on_page_context({}, page)
    assert context_after.get('footermatter_created') == '1. februar 2022 12:00'

    plugin.config['locale'] = 'en'
    page = Page({'created': '2022-02-01 12:00'})
    context_after = plugin.on_page_context({}, page)
    assert context_after.get('footermatter_created') == 'February 1, 2022 12:00 PM'

    # Custom
    plugin.config['locale'] = 'en'
    plugin.config['date_format'] = 'HH:mm:ss'
    page = Page({'created': '2022-02-01 12:00'})
    context_after = plugin.on_page_context({}, page)
    assert context_after.get('footermatter_created') == '12:00:00'

    plugin.config['date_format'] = 'YY-YYYY-MM-mm'
    page = Page({'created': '2022-02-01 12:30'})
    context_after = plugin.on_page_context({}, page)
    assert context_after.get('footermatter_created') == '22-2022-02-30'

    # Timeago
    plugin.config['date_format'] = 'timeago'
    page = Page({'created': pendulum.now().subtract(months=30, seconds=1)})
    context_after = plugin.on_page_context({}, page)
    assert context_after.get('footermatter_created') == '2 years'

    page = Page({'created': pendulum.now().subtract(months=8, seconds=1)})
    context_after = plugin.on_page_context({}, page)
    assert context_after.get('footermatter_created') == '8 months'

    page = Page({'created': pendulum.now().subtract(weeks=2, seconds=1)})
    context_after = plugin.on_page_context({}, page)
    assert context_after.get('footermatter_created') == '2 weeks'

    page = Page({'created': pendulum.now().subtract(hours=4, seconds=1)})
    context_after = plugin.on_page_context({}, page)
    assert context_after.get('footermatter_created') == '4 hours'

    page = Page({'created': pendulum.now().subtract(minutes=30, seconds=1)})
    context_after = plugin.on_page_context({}, page)
    assert context_after.get('footermatter_created') == '30 minutes'

    page = Page({'created': pendulum.now().subtract(days=2, seconds=1)})
    context_after = plugin.on_page_context({}, page)
    assert context_after.get('footermatter_created') == '2 days'

    page = Page({'created': pendulum.now().subtract(seconds=5)})
    context_after = plugin.on_page_context({}, page)
    assert context_after.get('footermatter_created') == 'a few seconds'



import pendulum

from dateutil import parser
from mkdocs.plugins import BasePlugin
from mkdocs.config import config_options

# TODO: Add more aliases
LOCALE_ALIASES = {
    'nb_NO': 'nb',
    'nb_NN': 'nn',
    'no': 'nb',
    'en_US': 'en'
}


class Author:
    def __init__(self, name, img, url):
        self.name = name
        self.img = img
        self.url = url


class FootermatterPlugin(BasePlugin):
    config_scheme = (
        ('key_authors', config_options.Type(str, default='authors')),
        ('key_created', config_options.Type(str, default='created')),
        ('key_updated', config_options.Type(str, default='updated')),
        ("locale", config_options.Type(str, default='')),
        ("date_format", config_options.Type(str, default='date')),
        ('timeago_absolute', config_options.Type(bool, default=True)),
        ('author_map', config_options.Type(list, default=[])),
        ("separator_map", config_options.Type(str, default='|')),
        ("default_author_img", config_options.Type(str, default='https://ui-avatars.com')),
        ("default_author_url", config_options.Type(str, default='/')),
    )

    def __init__(self):
        self.author_map = {}
        self.now = pendulum.now()

    def on_config(self, config: config_options.Config):
        """Define locale (from theme or specified) and authors from mkdocs.yml"""
        self.config['locale'] = str(self.get_locale(config))
        if self.config['locale'] in LOCALE_ALIASES.keys():  # pragma: no cover
            self.config['locale'] = LOCALE_ALIASES.get(self.config['locale'])

        self.author_map = {name.strip(): Author(name.strip(), img.strip(), url.strip())
                           for name, img, url in [author.split(self.config['separator_map'])
                                                  for author in self.config['author_map']]}

    def on_page_context(self, context, page, **kwargs):
        """Generate context values for the template, but only if all values exist"""
        authors, created, updated = self.get_frontmatter_keys(page)
        context['footermatter_authors'] = [self.get_author(a) for a in authors] if authors else None
        context['footermatter_created'] = self.format_date(created) if created else None
        context['footermatter_updated'] = self.format_date(updated) if updated else None
        return context

    ###
    # Util functions
    ###
    def get_locale(self, config: config_options.Config, fallback='en'):
        """Returns the locale in the given priority (specified, theme language, theme locale, en)"""
        if self.config.get('locale'):
            return self.config.get('locale')
        elif "theme" in config and "locale" in config.get("theme"):
            return config.get("theme")._vars.get("locale", fallback)
        elif "theme" in config and "language" in config.get("theme"):
            return config.get("theme")._vars.get("language", fallback)
        return fallback

    def get_author(self, author) -> Author:
        """Returns either a defined author from author_map, or creates a new one using default values"""
        if author not in self.author_map:
            return Author(author, self.config.get('default_author_img'), self.config.get('default_author_url'))
        return self.author_map.get(author)

    def get_frontmatter_keys(self, page) -> tuple:
        """Returns a tuple for the fronmatter values of authors, created and updated"""
        authors = page.meta.get(self.config.get("key_authors"))
        if authors is not None:  # Ensure authors is list
            authors = authors if isinstance(authors, list) else [authors]
        created = page.meta.get(self.config.get("key_created"))
        updated = page.meta.get(self.config.get("key_updated"))
        return authors, created, updated

    def format_timeago(self, diff) -> str:
        abslt = self.config.get('timeago_absolute')
        if diff.years:
            return self.now.subtract(years=diff.years).diff_for_humans(absolute=abslt)
        if diff.months:
            return self.now.subtract(months=diff.months).diff_for_humans(absolute=abslt)
        if diff.weeks:
            return self.now.subtract(weeks=diff.weeks).diff_for_humans(absolute=abslt)
        if diff.days:
            return self.now.subtract(days=diff.days).diff_for_humans(absolute=abslt)
        if diff.hours:
            return self.now.subtract(hours=diff.hours).diff_for_humans(absolute=abslt)
        if diff.minutes:
            return self.now.subtract(minutes=diff.minutes).diff_for_humans(absolute=abslt)
        return self.now.subtract(seconds=diff.seconds).diff_for_humans(absolute=abslt)


    def format_date(self, date):
        """Takes a date value and formats it to the given format"""
        if date is None:  # pragma: no cover
            return

        df, locale = self.config.get('date_format'), self.config.get('locale')
        date = parser.parse(date) if isinstance(date, str) else date
        date = pendulum.instance(date, tz=self.now.timezone)

        try:
            pendulum.set_locale(locale)
        except ValueError:  # pragma: no cover
            print(f'[WARNING]: Locale for mkdocs-footermatter is invalid (Got: {locale})')
            print(f'[WARNING]: Locale for mkdocs-footermatter is now set to \'en\'.')
            pendulum.set_locale('en')

        options = {
            'timeago': self.format_timeago(self.now - date),
            'date': date.format(fmt="LL"),
            'datetime': date.format(fmt='LLL')
        }

        return options.get(df, date.format(fmt=df))



from importlib.metadata import version, PackageNotFoundError

from mkdocs_footermatter.plugin import FootermatterPlugin

try:
    __version__ = version('mkdocs-footermatter')
except PackageNotFoundError:  # pragma: no cover
    # package is not installed
    pass